/***************************************************************************
 *   Copyright (C) 2004 by Heiko Schaefer                                  *
 *   heiko@rangun.de                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kdebug.h>

#include <qimage.h>
#include <qcstring.h>
#include <qmutex.h>

#include <kpixmap.h>

#include "wmfthumbcreator.h"

/**
 
$Id: wmfthumbcreator.cpp,v 1.3 2004/03/18 18:16:09 zxmpl18 Exp $

$Log: wmfthumbcreator.cpp,v $
Revision 1.3  2004/03/18 18:16:09  zxmpl18
* Bumped to version 0.2

Revision 1.2  2004/03/18 15:33:19  zxmpl18
* Adaptions to make it more KDE-conform


*/

QValueVector<char> WMFThumbCreator::m_imageData(AVGIMGSIZE);
unsigned long      WMFThumbCreator::m_flags =
	WMF_OPT_FUNCTION|WMF_OPT_IGNORE_NONFATAL;

WMFThumbCreator::WMFThumbCreator()
 : ThumbCreator()
{
	m_options.function = wmf_gd_function;
}


WMFThumbCreator::~WMFThumbCreator() {
	m_imageData.clear();
}

/*!
    \fn WMFThumbCreator::create(const QString &path, int width, int height, QImage &img)
 */
bool WMFThumbCreator::create(const QString &path, int width, int height, QImage &img)
{

	wmfD_Rect   bbox;
	wmf_error_t error;
	
	m_imageData.clear();
	
	if(wmf_api_create(&m_API, m_flags, &m_options) != wmf_E_None)
	{
		// is the API pointer now really valid?
		wmf_api_destroy(m_API);
		return false;
	}
	
	m_ddata = WMF_GD_GetData(m_API);

	if((m_ddata->flags & WMF_GD_SUPPORTS_PNG) == 0)
	{
		wmf_api_destroy(m_API);
		return false;
	}
	
	error = wmf_file_open(m_API, path.local8Bit());
	if(error != wmf_E_None)
	{
		wmf_api_destroy(m_API);
		return false;
	}

	error = wmf_scan(m_API, 0, &bbox);
	if(error != wmf_E_None)
	{
		wmf_file_close(m_API);
		wmf_api_destroy(m_API);
		return false;
	}

	m_ddata->type = wmf_gd_png;
	m_ddata->flags |= WMF_GD_OUTPUT_MEMORY|WMF_GD_OWN_BUFFER;
	m_ddata->sink.function = &sink;
	m_ddata->file = NULL;

	m_ddata->bbox.TL.x = bbox.TL.x * SCALE;
	m_ddata->bbox.TL.y = bbox.TL.y * SCALE;
	m_ddata->bbox.BR.x = bbox.BR.x * SCALE;
	m_ddata->bbox.BR.y = bbox.BR.y * SCALE;
	
	m_ddata->width  = (unsigned int)ceil(m_ddata->bbox.BR.x - m_ddata->bbox.TL.x);
	m_ddata->height = (unsigned int)ceil(m_ddata->bbox.BR.y - m_ddata->bbox.TL.y);
	
	error = wmf_play(m_API, 0, &m_ddata->bbox);
	if(error != wmf_E_None)
	{
		wmf_file_close(m_API);
		wmf_api_destroy(m_API);
		return false;
	}
	
	wmf_file_close(m_API);
	wmf_api_destroy(m_API);
	
	QByteArray ba(m_imageData.count());
	QValueVector<char>::iterator  iter;
	int i = 0;
	
	for(iter  = m_imageData.begin();
	    iter != m_imageData.end(); ++iter) {
		ba[i++] = *iter;
	}
	
	img.loadFromData(ba);

    return true;
}

/*!
    \fn WMFThumbCreator::sink(void * context, char * buffer, int length)
 */
int WMFThumbCreator::sink(void * context, char * buffer, int length)
{
	for(int i = 0; i < length; i++) {
		m_imageData.push_back(buffer[i]);
	}
	
    return m_imageData.count();
}
